@@ -8,7 +8,6 @@
 #undef ENABLE_IPV6
 #undef ENABLE_NLS
 #undef ALWAYS_RESTART_SERVER
-#undef GDM_USER_PATH
 #undef HAVE_ADT
 #undef HAVE_CATGETS
 #undef HAVE_CHKAUTHATTR
@@ -16,7 +15,6 @@
 #undef HAVE_CLEARENV
 #undef HAVE_CRYPT
 #undef HAVE_DEFOPEN
-#undef HAVE_FBCONSOLE
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_LIBSM
