@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include <errno.h>
+#include <limits.h>
 #include <pwd.h>
 #include <string.h>
 #include <sys/types.h>
@@ -220,13 +221,38 @@ _get_uid_and_gid_for_user (const char *username,
         return TRUE;
 }
 
+static void
+clean_up_stale_auth_subdirs (void)
+{
+        GDir *dir;
+        const char *filename;
+
+        dir = g_dir_open (GDM_XAUTH_DIR, 0, NULL);
+
+        if (dir == NULL) {
+                return;
+        }
+
+        while ((filename = g_dir_read_name (dir)) != NULL) {
+                char *path;
+
+                path = g_build_filename (GDM_XAUTH_DIR, filename, NULL);
+
+                /* Will only succeed if the directory is empty
+                 */
+                g_rmdir (path);
+                g_free (path);
+        }
+        g_dir_close (dir);
+}
+
 static FILE *
 _create_xauth_file_for_user (const char  *username,
                              char       **filename,
                              GError     **error)
 {
         char   *template;
-        char   *dir_name;
+        const char *dir_name;
         char   *auth_filename;
         int     fd;
         FILE   *fp;
@@ -262,6 +288,9 @@ _create_xauth_file_for_user (const char  *username,
         } else {
                 /* if it does exist make sure it has correct mode 01775 */
                 g_chmod (GDM_XAUTH_DIR, S_ISVTX | S_IRWXU |S_IRWXG | S_IROTH | S_IXOTH);
+
+                /* and clean up any stale auth subdirs */
+                clean_up_stale_auth_subdirs ();
         }
 
         if (!_get_uid_and_gid_for_user (username, &uid, &gid)) {
@@ -409,8 +438,13 @@ _get_auth_info_for_display (GdmDisplayAccessFile *file,
         gdm_display_is_local (display, &is_local, NULL);
 
         if (is_local) {
+                char localhost[HOST_NAME_MAX + 1] = "";
                 *family = FamilyLocal;
-                *address = g_strdup (g_get_host_name ());
+                if (gethostname (localhost, HOST_NAME_MAX) == 0) {
+                        *address = g_strdup (localhost);
+                } else {
+                        *address = g_strdup ("localhost");
+                }
         } else {
                 *family = FamilyWild;
                 gdm_display_get_remote_hostname (display, address, NULL);
