@@ -637,16 +637,6 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
 
                 gdm_session_auditor_set_username (worker->priv->auditor, worker->priv->username);
 
-                /* We have a new username to try. If we haven't been able to
-                 * read user settings up until now, then give it a go now
-                 * (see the comment in do_setup for rationale on why it's useful
-                 * to keep trying to read settings)
-                 */
-                if (username != NULL &&
-                    !gdm_session_settings_is_loaded (worker->priv->user_settings)) {
-                        attempt_to_load_user_settings (worker, username);
-                }
-
                 if ((worker->priv->username == username) ||
                     ((worker->priv->username != NULL) && (username != NULL) &&
                      (strcmp (worker->priv->username, username) == 0)))
@@ -661,6 +651,16 @@ gdm_session_worker_update_username (GdmSessionWorker *worker)
                 send_dbus_string_method (worker->priv->connection,
                                          "UsernameChanged",
                                          worker->priv->username);
+
+                /* We have a new username to try. If we haven't been able to
+                 * read user settings up until now, then give it a go now
+                 * (see the comment in do_setup for rationale on why it's useful
+                 * to keep trying to read settings)
+                 */
+                if (worker->priv->username != NULL &&
+                    !gdm_session_settings_is_loaded (worker->priv->user_settings)) {
+                        attempt_to_load_user_settings (worker, worker->priv->username);
+                }
         }
 
  out:
@@ -978,6 +978,12 @@ gdm_session_worker_uninitialize_pam (GdmSessionWorker *worker,
                 pam_close_session (worker->priv->pam_handle, 0);
                 gdm_session_auditor_report_logout (worker->priv->auditor);
         } else {
+                void *p;
+
+                if ((pam_get_item (worker->priv->pam_handle, PAM_USER, &p)) == PAM_SUCCESS) {
+                        gdm_session_auditor_set_username (worker->priv->auditor, (const char *)p);
+                }
+
                 gdm_session_auditor_report_login_failure (worker->priv->auditor,
                                                           status,
                                                           pam_strerror (worker->priv->pam_handle, status));
@@ -1000,7 +1006,11 @@ static char *
 _get_tty_for_pam (const char *x11_display_name,
                   const char *display_device)
 {
+#ifdef __sun
+        return g_strdup (display_device);
+#else
         return g_strdup (x11_display_name);
+#endif
 }
 
 #ifdef PAM_XAUTHDATA
