@@ -225,7 +225,10 @@ gdm_language_option_widget_init (GdmLanguageOptionWidget *widget)
 
         gdm_option_widget_add_item (GDM_OPTION_WIDGET (widget),
                                     "__other",
-                                    _("Other..."),
+                                    /* translators: This brings up a dialog
+                                     * with a list of languages to choose from
+                                     */
+                                    C_("language", "Other..."),
                                     _("Choose a language from the "
                                       "full list of available languages."),
                                     GDM_OPTION_WIDGET_POSITION_BOTTOM);
@@ -298,7 +301,7 @@ gdm_language_option_widget_set_current_language_name (GdmLanguageOptionWidget *w
 
         if (normalized_language_name != NULL &&
             !gdm_option_widget_lookup_item (GDM_OPTION_WIDGET (widget),
-                                            &normalized_language_name, NULL, NULL, NULL)) {
+                                            normalized_language_name, NULL, NULL, NULL)) {
                 gdm_recent_option_widget_add_item (GDM_RECENT_OPTION_WIDGET (widget),
                                                    normalized_language_name);
         }
